@@ -1528,5 +1528,5 @@ pub fn add_outer_product_broadcasts_on_top(add: &Add) -> Option<Add> {
             .rc()
         })
         .collect();
-    return Some(Add::try_new(new_operands, None).unwrap());
+    Some(Add::try_new(new_operands, None).unwrap())
 }
