@@ -14,7 +14,7 @@ use super::{
     deep_map_unwrap, visit_circuit, ArrayConstant, Circuit, CircuitNode, CircuitRc, HashBytes,
     Index,
 };
-use crate::{tensor_util::TorchDeviceDtype, util::AsOp};
+use crate::tensor_util::TorchDeviceDtype;
 
 #[pyfunction]
 pub fn cast_circuit(circ: CircuitRc, device_dtype: &TorchDeviceDtype) -> CircuitRc {
@@ -104,7 +104,7 @@ pub fn get_leaves(circuit: CircuitRc) -> Vec<CircuitRc> {
 #[pyfunction]
 pub fn get_all_einsum_specs(circuit: CircuitRc) -> Vec<EinsumSpec> {
     let mut result: Vec<EinsumSpec> = vec![];
-    visit_circuit(&**circuit, &mut |c| {
+    visit_circuit(&circuit, &mut |c| {
         if let Circuit::Einsum(node) = c {
             result.push(node.get_spec());
         }
@@ -156,7 +156,7 @@ where
 
 #[pyfunction]
 pub fn replace_all_randn_seeded(circuit: CircuitRc) -> CircuitRc {
-    deep_map_unwrap(&**circuit, &mut |node| match node {
+    deep_map_unwrap(&circuit, &mut |node| match node {
         Circuit::ArrayConstant(ac) => ArrayConstant::randn_named_seeded(
             ac.info().shape.clone(),
             ac.name_cloned(),
