@@ -28,7 +28,7 @@ use crate::{
     rearrange_spec::RearrangeSpec, // TODO: use new rearrange spec
     tensor_util::{
         broadcast_shapes, Slice, TensorAxisIndex, TensorIndex, TensorInvariantError,
-        TorchDeviceDtype, TorchDeviceDtypeOp,
+        TorchDeviceDtype,
     },
     util::{dedup_with_order, dict_to_list},
     util::{hashmap_collect_except_duplicates, EinInt},
@@ -94,17 +94,23 @@ impl CircuitNode for Einsum {
         Box::new(self.args.iter().map(|a| a.0.clone()))
     }
 
-    fn map_children<'a, F, E>(&'a self, f: &mut F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate<'a, F, E>(
+        &'a self,
+        f: &mut F,
+    ) -> Result<Self, CircuitConstructionError>
     where
         CircuitConstructionError: From<E>,
-        F: FnMut(&'a Circuit) -> Result<CircuitRc, E>,
+        F: FnMut(usize, &'a Circuit) -> Result<CircuitRc, E>,
     {
         Self::try_new(
             self.args
                 .iter()
-                .map(move |(circ, axes)| -> Result<_, CircuitConstructionError> {
-                    Ok((f(circ)?, axes.clone()))
-                })
+                .enumerate()
+                .map(
+                    move |(i, (circ, axes))| -> Result<_, CircuitConstructionError> {
+                        Ok((f(i, circ)?, axes.clone()))
+                    },
+                )
                 .collect::<Result<_, _>>()?,
             self.out_axes.clone(),
             self.name.clone(),
@@ -522,15 +528,19 @@ impl CircuitNode for Add {
         Box::new(self.nodes.iter().cloned())
     }
 
-    fn map_children<'a, F, E>(&'a self, f: &mut F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate<'a, F, E>(
+        &'a self,
+        f: &mut F,
+    ) -> Result<Self, CircuitConstructionError>
     where
         CircuitConstructionError: From<E>,
-        F: FnMut(&'a Circuit) -> Result<CircuitRc, E>,
+        F: FnMut(usize, &'a Circuit) -> Result<CircuitRc, E>,
     {
         Self::try_new(
             self.nodes
                 .iter()
-                .map(move |circ| f(circ))
+                .enumerate()
+                .map(move |(i, circ)| f(i, circ))
                 .collect::<Result<Vec<_>, _>>()?,
             self.name.clone(),
         )
@@ -696,12 +706,15 @@ impl CircuitNode for Rearrange {
         Box::new(std::iter::once(self.node.clone()))
     }
 
-    fn map_children<'a, F, E>(&'a self, f: &mut F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate<'a, F, E>(
+        &'a self,
+        f: &mut F,
+    ) -> Result<Self, CircuitConstructionError>
     where
         CircuitConstructionError: From<E>,
-        F: FnMut(&'a Circuit) -> Result<CircuitRc, E>,
+        F: FnMut(usize, &'a Circuit) -> Result<CircuitRc, E>,
     {
-        Self::try_new(f(&self.node)?, self.spec.clone(), self.name.clone())
+        Self::try_new(f(0, &self.node)?, self.spec.clone(), self.name.clone())
     }
 
     fn eval_tensors(
@@ -846,12 +859,15 @@ impl CircuitNode for Index {
         Box::new(std::iter::once(self.node.clone()))
     }
 
-    fn map_children<'a, F, E>(&'a self, f: &mut F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate<'a, F, E>(
+        &'a self,
+        f: &mut F,
+    ) -> Result<Self, CircuitConstructionError>
     where
         CircuitConstructionError: From<E>,
-        F: FnMut(&'a Circuit) -> Result<CircuitRc, E>,
+        F: FnMut(usize, &'a Circuit) -> Result<CircuitRc, E>,
     {
-        Self::try_new(f(&self.node)?, self.index.clone(), self.name.clone())
+        Self::try_new(f(0, &self.node)?, self.index.clone(), self.name.clone())
     }
 
     fn eval_tensors(
@@ -970,15 +986,19 @@ impl CircuitNode for Concat {
         Box::new(self.nodes.iter().cloned())
     }
 
-    fn map_children<'a, F, E>(&'a self, f: &mut F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate<'a, F, E>(
+        &'a self,
+        f: &mut F,
+    ) -> Result<Self, CircuitConstructionError>
     where
         CircuitConstructionError: From<E>,
-        F: FnMut(&'a Circuit) -> Result<CircuitRc, E>,
+        F: FnMut(usize, &'a Circuit) -> Result<CircuitRc, E>,
     {
         Self::try_new(
             self.nodes
                 .iter()
-                .map(move |circ| f(circ))
+                .enumerate()
+                .map(move |(i, circ)| f(i, circ))
                 .collect::<Result<Vec<_>, _>>()?,
             self.axis,
             self.name.clone(),
@@ -1156,13 +1176,16 @@ impl CircuitNode for Scatter {
             .collect()]
     }
 
-    fn map_children<'a, F, E>(&'a self, f: &mut F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate<'a, F, E>(
+        &'a self,
+        f: &mut F,
+    ) -> Result<Self, CircuitConstructionError>
     where
         CircuitConstructionError: From<E>,
-        F: FnMut(&'a Circuit) -> Result<CircuitRc, E>,
+        F: FnMut(usize, &'a Circuit) -> Result<CircuitRc, E>,
     {
         Self::try_new(
-            f(&self.node)?,
+            f(0, &self.node)?,
             self.index.clone(),
             self.info().shape.clone(),
             self.name.clone(),
