@@ -28,10 +28,13 @@ macro_rules! circuit_node_auto_leaf_impl {
             vec![]
         }
 
-        fn map_children<'a, F, E>(&'a self, _f: &mut F) -> Result<Self, CircuitConstructionError>
+        fn map_children_enumerate<'a, F, E>(
+            &'a self,
+            _f: &mut F,
+        ) -> Result<Self, CircuitConstructionError>
         where
             CircuitConstructionError: From<E>,
-            F: FnMut(&'a Circuit) -> Result<CircuitRc, E>,
+            F: FnMut(usize, &'a Circuit) -> Result<CircuitRc, E>,
         {
             Ok(self.clone())
         }
