@@ -299,7 +299,8 @@ pub fn add_pull_diags(add: &Add) -> Option<Einsum> {
                         if *l > 1 {
                             x + *rank_difference as u8
                         } else {
-                            (i + rank_difference + einsum.info().rank()) as u8
+                            (i + *rank_difference + *einsum.out_axes.iter().max().unwrap() as usize)
+                                as u8
                         }
                     },
                 ))
