@@ -1,10 +1,9 @@
 use crate::pyo3_prelude::*;
 use std::{collections::HashMap, iter::zip};
 
-use super::circuit_optimizer::optimize_and_evaluate;
 use super::{
-    Add, ArrayConstant, CachedCircuitInfo, Circuit, CircuitConstructionError, CircuitNode,
-    CircuitNodeAutoName, CircuitRc, Einsum, HashBytes, ScalarConstant,
+    CachedCircuitInfo, Circuit, CircuitConstructionError, CircuitNode, CircuitNodeAutoName,
+    CircuitRc, HashBytes,
 };
 use crate::py_types::{GENERALFUNCTIONS, PY_NONE};
 use crate::{
@@ -13,7 +12,7 @@ use crate::{
     py_types::py_address,
     tensor_util::{Shape, TorchDeviceDtype},
 };
-use cached::proc_macro::cached;
+
 use lazy_static::lazy_static;
 use macro_rules_attribute::apply;
 use pyo3::types::PyDict;
@@ -247,15 +246,19 @@ impl CircuitNode for GeneralFunction {
         Box::new(self.nodes.iter().cloned())
     }
 
-    fn map_children<'a, F, E>(&'a self, f: &mut F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate<'a, F, E>(
+        &'a self,
+        f: &mut F,
+    ) -> Result<Self, CircuitConstructionError>
     where
         CircuitConstructionError: From<E>,
-        F: FnMut(&'a Circuit) -> Result<CircuitRc, E>,
+        F: FnMut(usize, &'a Circuit) -> Result<CircuitRc, E>,
     {
         Self::try_new(
             self.nodes
                 .iter()
-                .map(move |circ| f(circ))
+                .enumerate()
+                .map(move |(i, circ)| f(i, circ))
                 .collect::<Result<Vec<_>, _>>()?,
             self.spec.clone(),
             self.name.clone(),
@@ -339,7 +342,7 @@ impl GeneralFunction {
         spec_name: String,
         name: Option<String>,
     ) -> Result<Self, CircuitConstructionError> {
-        let spec = GENERAL_FUNCTIONS_SPECS.get(&spec_name.clone());
+        let spec = GENERAL_FUNCTIONS_SPECS.get(&spec_name);
         let spec = spec.ok_or(CircuitConstructionError::UnknownGeneralFunction { spec_name })?;
         GeneralFunction::try_new(nodes, spec.clone(), name)
     }
