@@ -16,6 +16,7 @@ use thiserror::Error;
 pub mod algebraic_rewrite;
 pub mod batching;
 pub mod canonicalize;
+pub mod circuit_manipulation;
 pub mod circuit_optimizer;
 pub mod circuit_utils;
 pub mod compiler_heuristics;
@@ -113,11 +114,32 @@ pub trait CircuitNode: CircuitNodePrivate {
 
     fn children<'a>(&'a self) -> Box<dyn Iterator<Item = CircuitRc> + 'a>;
 
+    fn map_children_enumerate<'a, F, E>(
+        &'a self,
+        f: &mut F,
+    ) -> Result<Self, CircuitConstructionError>
+    where
+        Self: Sized,
+        CircuitConstructionError: From<E>,
+        F: FnMut(usize, &'a Circuit) -> Result<CircuitRc, E>;
+
     fn map_children<'a, F, E>(&'a self, f: &mut F) -> Result<Self, CircuitConstructionError>
     where
         Self: Sized,
         CircuitConstructionError: From<E>,
-        F: FnMut(&'a Circuit) -> Result<CircuitRc, E>;
+        F: FnMut(&'a Circuit) -> Result<CircuitRc, E>,
+    {
+        self.map_children_enumerate(&mut |_i, x| f(x))
+    }
+
+    fn map_children_idxs<'a, F, E>(&'a self, f: &mut F) -> Result<Self, CircuitConstructionError>
+    where
+        Self: Sized,
+        CircuitConstructionError: From<E>,
+        F: FnMut(usize) -> Result<CircuitRc, E>,
+    {
+        self.map_children_enumerate(&mut |i, _x| f(i))
+    }
 
     fn map_children_unwrap<'a, F>(&'a self, f: &mut F) -> Self
     where
@@ -128,6 +150,24 @@ pub trait CircuitNode: CircuitNodePrivate {
             .unwrap()
     }
 
+    fn map_children_unwrap_enumerate<'a, F>(&'a self, f: &mut F) -> Self
+    where
+        Self: Sized,
+        F: FnMut(usize, &'a Circuit) -> CircuitRc,
+    {
+        self.map_children_enumerate(&mut |i, x| Ok::<CircuitRc, CircuitConstructionError>(f(i, x)))
+            .unwrap()
+    }
+
+    fn map_children_unwrap_idxs<'a, F>(&'a self, f: &mut F) -> Self
+    where
+        Self: Sized,
+        F: FnMut(usize) -> CircuitRc,
+    {
+        self.map_children_enumerate(&mut |i, _x| Ok::<CircuitRc, CircuitConstructionError>(f(i)))
+            .unwrap()
+    }
+
     /// if any return Some, return child mapped, otherwise None
     fn map_children_op<'a, F>(&'a self, f: &mut F) -> Option<Self>
     where
@@ -227,11 +267,14 @@ pub trait CircuitNodeAutoName: CircuitNode {
 pub trait CircuitNodeUnion {
     fn as_trait_obj(&self) -> &dyn CircuitNode;
     fn as_trait_obj_mut(&mut self) -> &mut dyn CircuitNode;
-    fn map_children_impl<'a, F, E>(&'a self, f: &mut F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate_impl<'a, F, E>(
+        &'a self,
+        f: &mut F,
+    ) -> Result<Self, CircuitConstructionError>
     where
         Self: Sized,
         CircuitConstructionError: From<E>,
-        F: FnMut(&'a Circuit) -> Result<CircuitRc, E>;
+        F: FnMut(usize, &'a Circuit) -> Result<CircuitRc, E>;
     fn variant_string(&self) -> String;
     fn node_type_uuid(&self) -> [u8; 16];
 }
@@ -359,14 +402,14 @@ macro_rules! define_circuit_union_impl {
                 }
             }
 
-            fn map_children_impl<'a, F, E>(&'a self, f: &mut F) -> Result<Self, $crate::circuit::CircuitConstructionError>
+            fn map_children_enumerate_impl<'a, F, E>(&'a self, f: &mut F) -> Result<Self, $crate::circuit::CircuitConstructionError>
             where
                 $crate::circuit::CircuitConstructionError: From<E>,
-                F: FnMut(&'a $crate::circuit::Circuit) -> Result<$crate::circuit::CircuitRc, E>,
+                F: FnMut(usize,&'a $crate::circuit::Circuit) -> Result<$crate::circuit::CircuitRc, E>,
             {
                 match self {
                     $(
-                        Self::$x(node) => $crate::circuit::CircuitNode::map_children(node, f).map(|v| Self::$x(v)),
+                        Self::$x(node) => $crate::circuit::CircuitNode::map_children_enumerate(node, f).map(|v| Self::$x(v)),
                     )*
                 }
             }
@@ -587,12 +630,15 @@ impl<T: CircuitNodeUnion> CircuitNode for T {
         self.as_trait_obj().children()
     }
 
-    fn map_children<'a, F, E>(&'a self, f: &mut F) -> Result<Self, CircuitConstructionError>
+    fn map_children_enumerate<'a, F, E>(
+        &'a self,
+        f: &mut F,
+    ) -> Result<Self, CircuitConstructionError>
     where
         CircuitConstructionError: From<E>,
-        F: FnMut(&'a Circuit) -> Result<CircuitRc, E>,
+        F: FnMut(usize, &'a Circuit) -> Result<CircuitRc, E>,
     {
-        self.map_children_impl(f)
+        self.map_children_enumerate_impl(f)
     }
 
     fn self_flops(&self) -> BigUint {
@@ -647,7 +693,7 @@ impl CachedCircuitInfo {
     }
 }
 
-#[derive(Error, Debug)]
+#[derive(Error, Debug, Clone)]
 pub enum CircuitConstructionError {
     #[error("len shape different from len axes (len axes: {len_axes}, len shape: {circuit_len_shape}, circuit name: {circuit_name:?})")]
     EinsumLenShapeDifferentFromAxes {
@@ -718,7 +764,7 @@ pub enum CircuitConstructionError {
     UnknownGeneralFunction { spec_name: String },
 
     #[error("python error {py_err:?}")]
-    PythonError { py_err: PyErr },
+    PythonError { py_err: Rc<PyErr> },
 }
 
 impl From<CircuitConstructionError> for PyErr {
@@ -729,7 +775,9 @@ impl From<CircuitConstructionError> for PyErr {
 
 impl From<PyErr> for CircuitConstructionError {
     fn from(py_err: PyErr) -> Self {
-        CircuitConstructionError::PythonError { py_err }
+        CircuitConstructionError::PythonError {
+            py_err: Rc::new(py_err),
+        }
     }
 }
 
@@ -825,6 +873,32 @@ pub fn evaluate_fn(circ: &Circuit) -> Result<Tensor, TensorEvalError> {
     recurse(circ.clone())
 }
 
+pub fn deep_map<F>(circuit: &Circuit, f: &F) -> Result<CircuitRc, CircuitConstructionError>
+where
+    F: Fn(&Circuit) -> Result<CircuitRc, CircuitConstructionError>,
+{
+    cached_circuit_lambda! {
+        let mut recurse = |circ: &Circuit| -> Result<CircuitRc,CircuitConstructionError> = {
+            let inner_mapped =
+                circ.map_children(&mut recurse)?;
+            f(&inner_mapped)
+        };
+    }
+    recurse(circuit)
+}
+
+pub fn deep_map_preorder<F>(circuit: &Circuit, f: &F) -> Result<CircuitRc, CircuitConstructionError>
+where
+    F: Fn(&Circuit) -> Result<CircuitRc, CircuitConstructionError>,
+{
+    cached_circuit_lambda! {
+        let mut recurse = |circ: &Circuit| -> Result<CircuitRc,CircuitConstructionError> = {
+            f(circ)?.map_children(&mut recurse).map(|z|z.rc())
+        };
+    }
+    recurse(circuit)
+}
+
 pub fn deep_map_unwrap<F>(circuit: &Circuit, f: &F) -> CircuitRc
 where
     F: Fn(&Circuit) -> CircuitRc,
@@ -874,6 +948,28 @@ where
     recurse(circuit, &mut seen, f);
 }
 
+pub fn visit_circuit_fallable<F, E>(circuit: &Circuit, f: &mut F) -> Result<(), E>
+where
+    F: FnMut(&Circuit) -> Result<(), E>,
+{
+    let mut seen: HashSet<HashBytes> = HashSet::new();
+
+    fn recurse<F, E>(circ: &Circuit, seen: &mut HashSet<HashBytes>, f: &mut F) -> Result<(), E>
+    where
+        F: FnMut(&Circuit) -> Result<(), E>,
+    {
+        if !seen.contains(&circ.info().hash) {
+            seen.insert(circ.info().hash);
+            f(circ)?;
+            for child in circ.children() {
+                recurse(&child, seen, f)?
+            }
+        }
+        Ok(())
+    }
+    recurse(circuit, &mut seen, f)
+}
+
 pub fn visit_circuit_postorder<F>(circuit: &Circuit, f: &mut F)
 where
     F: FnMut(&Circuit),
