@@ -208,8 +208,8 @@ pub fn repr_circuit_one_level(circuit: &CircuitRc, context: &HashMap<CircuitRc,
 /// {
 ///     let node0 = ScalarConstant::new(0.2, vec![3], Some("a".to_owned())).rc();
 ///     let node1 = ScalarConstant::new(0.5, vec![3], Some("ScalarConstant".to_owned())).rc();
-///     let node2 = Add::nrc(vec![node0.clone(), node1.clone()], Some("s".to_owned())).rc();
-///     Concat::nrc(vec![node0.clone(), node2.clone()], 0, Some("cat".to_owned())).rc()
+///     let node2 = Add::nrc(vec![node0.clone(), node1.clone()], Some("s".to_owned()));
+///     Concat::nrc(vec![node0.clone(), node2.clone()], 0, Some("cat".to_owned()))
 /// };
 /// ```
 #[pyfunction]
@@ -241,7 +241,7 @@ pub fn rust_expression_notation_circuit(circuit: CircuitRc) -> String {
             source.push_str(format!("    {}\n", node_source).as_str());
         }
     }
-    source.push_str("}");
+    source.push('}');
     source
 }
 
