@@ -13,6 +13,10 @@ use circuit::algebraic_rewrite::{
 };
 use circuit::batching::{batch_inputs_axis_len, batch_largest};
 use circuit::canonicalize::{canonicalize_node_py, deep_canonicalize};
+use circuit::circuit_manipulation::{
+    deep_map_preorder_py, deep_map_py, filter_nodes_py, path_get, replace_nodes, update_nodes_py,
+    update_path_py,
+};
 use circuit::circuit_optimizer::{
     optimize_and_evaluate, optimize_and_evaluate_many, optimize_circuit,
 };
@@ -210,6 +214,14 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         m
     )?)?;
 
+    m.add_function(wrap_pyfunction!(deep_map_preorder_py, m)?)?;
+    m.add_function(wrap_pyfunction!(deep_map_py, m)?)?;
+    m.add_function(wrap_pyfunction!(filter_nodes_py, m)?)?;
+    m.add_function(wrap_pyfunction!(replace_nodes, m)?)?;
+    m.add_function(wrap_pyfunction!(update_nodes_py, m)?)?;
+    m.add_function(wrap_pyfunction!(path_get, m)?)?;
+    m.add_function(wrap_pyfunction!(update_path_py, m)?)?;
+
     // add more py function as desired or whatever...
     Ok(())
 }
