@@ -2,7 +2,7 @@ use crate::pyo3_prelude::*;
 use crate::{
     circuit::{CircuitConstructionError, HashBytes},
     filter_by_variant,
-    py_types::{Tensor, CAST_TENSOR, PY_CAST_INT, PY_ID, PY_TORCH},
+    py_types::{Tensor, CAST_TENSOR, PY_CAST_INT, PY_TORCH},
     pycall,
     rearrange_spec::RearrangeSpec,
 };
